/**********************************************************************
 * perdition_gdbm.c                                            May 1999
 * Horms                                            horms@vergenet.net.
 * Verge Systems International                    http://vergenet.net./
 *
 * Access a gdbm(3) database
 **********************************************************************/

#include "perdition_gdbm.h"


/**********************************************************************
 * gdbm_getserver
 * Read the server information for a given key from the gdbmmap
 * specified in the options structure.
 * Result is a pointer to a server_port_t structure that is
 * allocated by this function
 **********************************************************************/

server_port_t *gdbm_getserver(
  const perdition_options_t options,
  char * key_str
){
  GDBM_FILE dbf;
  datum key;
  datum content;
  server_port_t *server_port;
  
  key.dptr=key_str;
  key.dsize=strlen(key_str);

  if((dbf=gdbm_open(options.mapname, 0, GDBM_READER, 0644, 0))==NULL){
    syslog(
      LOG_WARNING, 
      "gdbm_getserver: gdbm_open: %s: %s\n", 
      gdbm_strerror(gdbm_errno), 
      strerror(errno)
    );
    return(NULL);
  }
  content=gdbm_fetch(dbf,key);
  gdbm_close(dbf);


  if((server_port=server_port_create(server_port))==NULL){
    syslog(LOG_WARNING, "gdbm_getserver: server_port_create");
    return(NULL);
  }
  
  if(content.dptr!=NULL){
    server_port=server_port_datum_assign(server_port, content);
  }
  else{
    return(NULL);
  }

  return(server_port);
} 
