/**********************************************************************
 * queue.c                                                   March 1999
 * Horms                                            horms@verge.ing.to.
 * Verge Systems International                http://www.verge.ing.to./
 *
 **********************************************************************/

#include "queue.h"

void free_queue_member(
  queue_member *mem, 
  void (*destroy_queue_t)(const void *)
);
 
queue *create_queue(queue *q){
  if( (q=(queue *)malloc(sizeof(queue))) == NULL ){
    syslog(
      LOG_WARNING, 
      "create_queue: could not allocate memory: %s",
      strerror(errno)
    );
    return(NULL);
  }
  
  q->first=NULL;
  q->last=NULL;
  q->size=0;

  return q;
}

queue *push(queue *q, queue_t value, void (*destroy_queue_t)(const void *)){
  queue_member *new;

  if( (new=(queue_member *)malloc(sizeof(queue_member))) == NULL){
    syslog(
      LOG_WARNING, 
      "push: could not allocate memory: %s",
      strerror(errno)
    );
    destroy_queue(q, (void (*)(const void *))destroy_queue_t);
    return(NULL);
  }

  /*Put in payload*/
  new->value=value;

  /*Update queue pointers*/
  new->next=q->first;
  new->prev=NULL;
  if(q->first==NULL){
    q->last=new;
  }
  else{
    q->first->prev = new;
  }
  q->first=new;

  /*Increment queue size*/
  q->size++;

  return q;
}

queue *pop(queue *q, queue_t *value){
  queue_member *old_last;

  if(q==NULL){
    syslog(
      LOG_WARNING, 
      "pop(): poped an NULL queue: %s",
      strerror(errno)
    );
    return(NULL);
  }

  if(q->last==NULL){
    syslog(
      LOG_WARNING, 
      "pop(): poped an empty queue: %s",
      strerror(errno)
    );
    return(NULL);
  }

  /*Grab payload*/
  *value = q->last->value;

  /*Fix pointers*/
  old_last = q->last;
  if ( (q->last=old_last->prev) == NULL ){
    q->first=NULL;
  }
  else{
    q->last->next=NULL;
  }
  
  /*Decrement queue size*/
  q->size--;

  free_queue_member(old_last, NULL);
  
  return q;
}

void destroy_queue(queue *q, void (*destroy_queue_t)(const void *)){
  queue_member *old_first;

  if(q==NULL){
    return;
  }

  while(q->first!=NULL){
    old_first = q->first;
    q->first = old_first->next;
    free_queue_member(old_first, (void (*)(const void *))destroy_queue_t);
  }

  free(q);
}

int queue_length(queue *q){
  if(q!=NULL){
    return q->size;
  }
  else{
    return -1;
  }
}
  
void free_queue_member(
  queue_member *mem, 
  void (*destroy_queue_t)(const void *)
){
  if(mem==NULL){
    return;
  }
/*
  if(mem->value!=NULL && destroy_queue_t!=NULL){
    destroy_queue_t(mem->value);
  }
  free(mem);
*/
}
