/**********************************************************************
 * server_port.c                                               May 1999
 * Horms                                            horms@verge.ing.to.
 * Verge Systems International                http://www.verge.ing.to./
 *
 **********************************************************************/

#include "server_port.h"


/**********************************************************************
 * server_port_create
 * Create an empty server_port structure
 **********************************************************************/

server_port_t *server_port_create (server_port_t *server_port){
  if((server_port=(server_port_t *)malloc(sizeof(server_port_t)))==NULL){
    syslog(
      LOG_WARNING, 
      "server_port_create: malloc: %s",
      strerror(errno)
    );
    return(NULL);
  }
  server_port->servername=NULL;
  server_port->port=NULL;
  return(server_port);
}


/**********************************************************************
 * server_port_assign
 * Assign values to a server_port_structure
 **********************************************************************/

void server_port_assign(
  server_port_t *server_port, 
  char *servername, 
  char *port
){
  server_port->servername=servername;
  server_port->port=port;
}


/**********************************************************************
 * server_port_datum_assign
 * Assign the data in a dataum data structure to a port structure
 *
 * dat.dvalue should be of the form 
 *  <servername>:<port>   or
 *  <servername>:         or
 *  <servername>
 *
 * In the case of the last two server_port_t.port will be set to NULL
 *
 * Altenatively if dat.ssize==0 both servername and port in
 * the resulting server_port_t will be NULL
 **********************************************************************/

server_port_t *server_port_datum_assign(server_port_t *server_port, datum dat){
  int delimiter;

  server_port_unassign(server_port);

  if(dat.dsize==0){
    return(server_port);
  }

  for(delimiter=0;delimiter<dat.dsize;delimiter++){
    if(*(dat.dptr+delimiter)==FIELD_DELIMITER){
      break;
    }
  }

  if(delimiter!=dat.dsize){
    if((server_port->port=strn_to_str(
      server_port->port, 
      dat.dptr+delimiter+1, 
      dat.dsize-delimiter-1
    ))==NULL){
      syslog( LOG_WARNING, "server_port_datum_assign: strn_to_str port");
      free(server_port);
      return(NULL);
    }
  }

  if((server_port->servername=strn_to_str(
    server_port->servername, 
    dat.dptr,
    delimiter
  ))==NULL){
    syslog(LOG_WARNING, "server_port_datum_assign: strn_to_str servername");
    free(server_port->port);
    free(server_port);
    return(NULL);
  }

  return(server_port);
}


/**********************************************************************
 * server_port_unassign
 * Unassign any values but do no free their memory.
 **********************************************************************/

void server_port_unassign(server_port_t *server_port){
  server_port_assign(server_port, (char *)NULL, (char *)NULL);
}


/**********************************************************************
 * server_port_unassign
 * Free a server_port structure and free any non NULL elements.
 *
 * If you don't want to free the memory  of elements you should
 * use server_port_unassign(server_port);
 * first.
 **********************************************************************/

void server_port_destroy(server_port_t *server_port){
  if(server_port==NULL){
    return;
  }

  if(server_port->servername!=NULL){
    free(server_port->servername);
  }
  if(server_port->port!=NULL){
    free(server_port->port);
  }

  free(server_port);
}


/**********************************************************************
 * server_port_get_port
 * Get the port from a server_port structure
 **********************************************************************/

char * server_port_get_port(server_port_t *server_port){
  return(server_port->port);
}


/**********************************************************************
 * server_port_get_servername
 * Get the servername from a server_port_structure
 **********************************************************************/

char * server_port_get_servername(server_port_t *server_port){
  return(server_port->servername);
}






